# Scrapped from: https://www.wynndata.tk/items/

major_ids = {
    "HERO": {
        "name": "Saviour's Sacrifice",
        "effect": "While under 50% maximum health, nearby allies gain 50% bonus damage and defence"
    },
    "MAGNET": {
        "name": "Magnet",
        "effect": "Pulls items within an 8 block radius towards you"
    },
    "LIGHTWEIGHT": {
        "name": "Lightweight",
        "effect": "You no longer take fall damage"
    },
    "ENTROPY": {
        "name": "Entropy",
        "effect": "Meteor falls three times faster"
    },
    "MADNESS": {
        "name": "Madness",
        "effect": "Casts a random ability every 3 seconds"
    },
    "EXPLOSIVE_IMPACT": {
        "name": "Explosive Impact",
        "effect": "Your \"Exploding\" ID can trigger whet hitting mobs with your basic attack"
    },
    "ARCANES": {
        "name": "Transcendence",
        "effect": "50% chance for spells to cost no mana when casted"
    },
    "PLAGUE": {
        "name": "Plague",
        "effect": "Poisoned mobs spread their poison to nearby mobs"
    },
    "RALLY": {
        "name": "Rally",
        "effect": "Charge heals you by 10% and nearby allies by 15% on impact, but becomes harmless"
    },
    "FISSION": {
        "name": "Fission",
        "effect": "Explosions from your \"Exploding\" ID are twice as big and twice as strong"
    },
    "TAUNT": {
        "name": "Taunt",
        "effect": "Mobs within 12 blocks target you upon casting War Scream"
    },
    "HAWKEYE": {
        "name": "Hawkeye",
        "effect": "Arrow storm fires 5 arrows each dealing 80% damage"
    },
    "ROVINGASSASSIN": {
        "name": "Roving Assassin",
        "effect": "Vanish no longer drains mana while invisible"
    },
    "ALTRUISM": {
        "name": "Heart of the Pack",
        "effect": "Nearby players gain 35% of the health you naturally regenerate"
    },
    "FLASHFREEZE": {
        "name": "Flashfreeze",
        "effect": "Ice Snake is instant and freezes for longer at a reduced range"
    },
    "FREERUNNER": {
        "name": "Freerunner",
        "effect": "Double your sprint speed when your sprint bar is under 30%"
    },
    "GREED": {
        "name": "Greed",
        "effect": "Picking up emeralds heals you and nearby players for 15% max health"
    },
    "CAVALRYMAN": {
        "name": "Cavalryman",
        "effect": "You may cast spells and attack with a 70% damage penalty while on a horse"
    },
    "SORCERY": {
        "name": "Sorcery",
        "effect": "30% chance for spells and attacks to cast a second time at no additional cost"
    },
    "GUARDIAN": {
        "name": "Guardian",
        "effect": "50% of damage taken by nearby allies is redirected to you"
    },
    "PEACEFUL_EFFIGY": {
        "name": "Peaceful Effigy",
        "effect": "Your totem will last twice as long"
    },
    "CHERRY_BOMBS": {
        "name": "Cherry Bombs",
        "effect": "Your Smoke Bombs explodes instantly on contact, dealing 110% damage each"
    },
    "FURIOUS_EFFIGY": {
        "name": "Furious Effigy",
        "effect": "Totem effects are twice as fast, but duration is halved"
    }
}
