# Translation of identification values returned by API
# to values used by library

from corkus.objects.identification_type import IdentificationType

ids_convert = [
    {
        "type": IdentificationType.AIR_DEFENSE,
        "items_api": "airDefense",
        "ingredient_api": ""
    }, {
        "type": IdentificationType.EARTH_DEFENSE,
        "items_api": "earthDefense",
        "ingredient_api": ""
    }, {
        "type": IdentificationType.FIRE_DEFENSE,
        "items_api": "fireDefense",
        "ingredient_api": ""
    }, {
        "type": IdentificationType.THUNDER_DEFENSE,
        "items_api": "thunderDefense",
        "ingredient_api": ""
    }, {
        "type": IdentificationType.WATER_DEFENSE,
        "items_api": "waterDefense",
        "ingredient_api": ""
    }, {
        "type": IdentificationType.AIR_DEFENSE_BONUS,
        "items_api": "bonusAirDefense",
        "ingredient_api": "AIRDEFENSE"
    }, {
        "type": IdentificationType.EARTH_DEFENSE_BONUS,
        "items_api": "bonusEarthDefense",
        "ingredient_api": "EARTHDEFENSE"
    }, {
        "type": IdentificationType.FIRE_DEFENSE_BONUS,
        "items_api": "bonusFireDefense",
        "ingredient_api": "FIREDEFENSE"
    }, {
        "type": IdentificationType.THUNDER_DEFENSE_BONUS,
        "items_api": "bonusThunderDefense",
        "ingredient_api": "THUNDERDEFENSE"
    }, {
        "type": IdentificationType.WATER_DEFENSE_BONUS,
        "items_api": "bonusWaterDefense",
        "ingredient_api": "WATERDEFENSE"
    }, {
        "type": IdentificationType.DAMAGE_BONUS,
        "items_api": "damageBonus",
        "ingredient_api": "DAMAGEBONUS"
    }, {
        "type": IdentificationType.DAMAGE_BONUS_RAW,
        "items_api": "damageBonusRaw",
        "ingredient_api": "DAMAGEBONUSRAW"
    }, {
        "type": IdentificationType.SPELL_DAMAGE,
        "items_api": "spellDamage",
        "ingredient_api": "SPELLDAMAGE"
    }, {
        "type": IdentificationType.SPELL_DAMAGE_RAW,
        "items_api": "spellDamageRaw",
        "ingredient_api": "SPELLDAMAGERAW"
    }, {
        "type": IdentificationType.AIR_DAMAGE_BONUS,
        "items_api": "bonusAirDamage",
        "ingredient_api": "AIRDAMAGEBONUS"
    }, {
        "type": IdentificationType.EARTH_DAMAGE_BONUS,
        "items_api": "bonusEarthDamage",
        "ingredient_api": "EARTHDAMAGEBONUS"
    }, {
        "type": IdentificationType.FIRE_DAMAGE_BONUS,
        "items_api": "bonusFireDamage",
        "ingredient_api": "FIREDAMAGEBONUS"
    }, {
        "type": IdentificationType.THUNDER_DAMAGE_BONUS,
        "items_api": "bonusThunderDamage",
        "ingredient_api": "THUNDERDAMAGEBONUS"
    }, {
        "type": IdentificationType.WATER_DAMAGE_BONUS,
        "items_api": "bonusWaterDamage",
        "ingredient_api": "WATERDAMAGEBONUS"
    }, {
        "type": IdentificationType.AGILITY_POINTS,
        "items_api": "agilityPoints",
        "ingredient_api": "AGILITYPOINTS"
    }, {
        "type": IdentificationType.DEFENSE_POINTS,
        "items_api": "defensePoints",
        "ingredient_api": "DEFENSEPOINTS"
    }, {
        "type": IdentificationType.DEXTERITY_POINTS,
        "items_api": "dexterityPoints",
        "ingredient_api": "DEXTERITYPOINTS"
    }, {
        "type": IdentificationType.INTELLIGENCE_POINTS,
        "items_api": "intelligencePoints",
        "ingredient_api": "INTELLIGENCEPOINTS"
    }, {
        "type": IdentificationType.STRENGTH_POINTS,
        "items_api": "strengthPoints",
        "ingredient_api": "STRENGTHPOINTS"
    }, {
        "type": IdentificationType.POISON,
        "items_api": "poison",
        "ingredient_api": "POISON"
    }, {
        "type": IdentificationType.MANA_STEAL,
        "items_api": "manaSteal",
        "ingredient_api": "MANASTEAL"
    }, {
        "type": IdentificationType.MANA_REGEN,
        "items_api": "manaRegen",
        "ingredient_api": "MANAREGEN"
    }, {
        "type": IdentificationType.WALK_SPEED,
        "items_api": "speed",
        "ingredient_api": "SPEED"
    }, {
        "type": IdentificationType.HEALTH_BONUS,
        "items_api": "healthBonus",
        "ingredient_api": "HEALTHBONUS"
    }, {
        "type": IdentificationType.HEALTH_REGEN,
        "items_api": "healthRegen",
        "ingredient_api": "HEALTHREGEN"
    }, {
        "type": IdentificationType.HEALTH_REGEN_RAW,
        "items_api": "healthRegenRaw",
        "ingredient_api": "HEALTHREGENRAW"
    }, {
        "type": IdentificationType.ATTACK_SPEED,
        "items_api": "attackSpeedBonus",
        "ingredient_api": "ATTACKSPEED"
    }, {
        "type": IdentificationType.LIFE_STEAL,
        "items_api": "lifeSteal",
        "ingredient_api": "LIFESTEAL"
    }, {
        "type": IdentificationType.REFLECTION,
        "items_api": "reflection",
        "ingredient_api": "REFLECTION"
    }, {
        "type": IdentificationType.THORNS,
        "items_api": "thorns",
        "ingredient_api": "THORNS"
    }, {
        "type": IdentificationType.EXPLODING,
        "items_api": "exploding",
        "ingredient_api": "EXPLODING"
    }, {
        "type": IdentificationType.LOOT_BONUS,
        "items_api": "lootBonus",
        "ingredient_api": "LOOTBONUS"
    }, {
        "type": IdentificationType.XP_BONUS,
        "items_api": "xpBonus",
        "ingredient_api": "XPBONUS"
    }, {
        "type": IdentificationType.EMERALD_STEALING,
        "items_api": "emeraldStealing",
        "ingredient_api": "EMERALDSTEALING"
    }, {
        "type": IdentificationType.GATHER_XP_BONUS,
        "items_api": "gatherXpBonus",
        "ingredient_api": "GATHER_XP_BONUS"
    }, {
        "type": IdentificationType.GATHER_SPEED,
        "items_api": "gatherSpeed",
        "ingredient_api": "GATHER_SPEED"
    }, {
        "type": IdentificationType.SOUL_POINTS,
        "items_api": "soulPoints",
        "ingredient_api": "SOULPOINTS"
    }, {
        "type": IdentificationType.SPRINT,
        "items_api": "sprintRegen",
        "ingredient_api": ""
    }, {
        "type": IdentificationType.SPRINT_REGEN,
        "items_api": "sprint",
        "ingredient_api": ""
    }, {
        "type": IdentificationType.JUMP_HEIGHT,
        "items_api": "jumpHeight",
        "ingredient_api": ""
    }, {
        "type": IdentificationType.LOOT_QUALITY,
        "items_api": "lootQuality",
        "ingredient_api": ""
    }, {
        "type": IdentificationType.RAINBOW_SPELL_DAMAGE_RAW,
        "items_api": "rainbowSpellDamageRaw",
        "ingredient_api": ""
    }, {
        "type": IdentificationType.SPELL_COST_1,
        "items_api": "spellCostRaw1",
        "ingredient_api": ""
    }, {
        "type": IdentificationType.SPELL_COST_1_RAW,
        "items_api": "spellCostPct1",
        "ingredient_api": ""
    }, {
        "type": IdentificationType.SPELL_COST_2,
        "items_api": "spellCostRaw2",
        "ingredient_api": ""
    }, {
        "type": IdentificationType.SPELL_COST_2_RAW,
        "items_api": "spellCostPct2",
        "ingredient_api": ""
    }, {
        "type": IdentificationType.SPELL_COST_3,
        "items_api": "spellCostRaw3",
        "ingredient_api": ""
    }, {
        "type": IdentificationType.SPELL_COST_3_RAW,
        "items_api": "spellCostPct3",
        "ingredient_api": ""
    }, {
        "type": IdentificationType.SPELL_COST_4,
        "items_api": "spellCostRaw5",
        "ingredient_api": ""
    }, {
        "type": IdentificationType.SPELL_COST_4_RAW,
        "items_api": "spellCostPct4",
        "ingredient_api": ""
    }
]
